package org.vraptor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.core.InterceptorsLogicFlow;
import org.vraptor.interceptor.InterceptorType;
import org.vraptor.view.ViewException;

/**
 * An interceptor that can be used to group common interceptors into one,
 * avoiding lots of classes inside an InterceptedBy annotation.
 * 
 * @author Paulo Silveira
 * 
 */
public class InterceptorStack implements Interceptor {

	private List<Interceptor> interceptors;

	/**
	 * Constructs an InterceptorStack with the given interceptors
	 * 
	 * @param interceptors
	 */
	public InterceptorStack(Class<? extends Interceptor>... interceptorClasses) {
		if (interceptorClasses == null) {
			throw new NullPointerException();
		}

		this.interceptors = new ArrayList<Interceptor>();

		for (Class<? extends Interceptor> interceptorClass : interceptorClasses) {
			InterceptorType type = InterceptorType.getType(interceptorClass);
			try {
				interceptors.add(type.newInstance());
			} catch (ComponentInstantiationException e) {
				throw new RuntimeException(e);
			}
		}
	}

	/**
	 * Intercepts the logic flow with the given interceptors, respecting the
	 * order.
	 */
	public void intercept(LogicFlow realFlow) throws LogicException, ViewException {
		InterceptorsLogicFlow flow = (InterceptorsLogicFlow) realFlow;
		for (Interceptor i : interceptors) {
			flow.addInterceptor(i);
		}
		realFlow.execute();
	}

	public List<Interceptor> getInterceptors() {
		return Collections.unmodifiableList(interceptors);
	}
}
