package org.vraptor.component;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.vraptor.LogicException;
import org.vraptor.LogicRequest;
import org.vraptor.annotations.Parameter;
import org.vraptor.annotations.Viewless;
import org.vraptor.reflection.JavaMethod;
import org.vraptor.reflection.MethodInvocationException;
import org.vraptor.validator.BasicValidationErrors;
import org.vraptor.validator.UnstableValidationException;
import org.vraptor.validator.ValidationErrors;

/**
 * Represents a logic method.
 * 
 * @author Guilherme Silveira
 */
public class DefaultLogicMethod implements LogicMethod {

	private static final Logger LOG = Logger.getLogger(DefaultLogicMethod.class);

	private final String name;

	private final JavaMethod method;

	private final boolean shouldRedirect;

	private final JavaMethod validateMethod;

	private final List<MethodParameter> parameters;

	/**
	 * Constructs it based on the logics name and method
	 * 
	 * @param method
	 *            method
	 */
	public DefaultLogicMethod(String name, Method method, Method validateMethod) {
		this.name = name;
		this.method = new JavaMethod(method);
		this.shouldRedirect = !this.method.containsAnnotation(Viewless.class);

		// TODO add null pattern as NullJavaMethod
		if (validateMethod != null) {
			this.validateMethod = new JavaMethod(validateMethod);
		} else {
			this.validateMethod = null;
		}

		this.parameters = loadParameters(method);

		LOG.debug("Logic method " + method.getName() + " was read with validation " + this.validateMethod);

	}

	private List<MethodParameter> loadParameters(Method method) {
		Class<?>[] params = method.getParameterTypes();
		Type[] generic = method.getGenericParameterTypes();
		Annotation[][] annotations = method.getParameterAnnotations();
		List<MethodParameter> list = new ArrayList<MethodParameter>();
		for (int i = 0; i < params.length; i++) {
			Class<?> param = params[i];
			Parameter found = searchParameterAnnotation(annotations, i);
			list.add(new MethodParameter(param, generic[i], i, found));
		}
		return list;
	}

	private Parameter searchParameterAnnotation(Annotation[][] annotations, int i) {
		for(Annotation a : annotations[i]) {
			if(a.annotationType().equals(Parameter.class)) {
				return (Parameter) a;
			}
		}
		return null;
	}

	public String getName() {
		return this.name;
	}

	public String execute(Object component, LogicRequest context, Object[] params) throws LogicException {
		Object result;
		try {
			result = method.invoke(component, params);
		} catch (MethodInvocationException e) {
			throw new LogicException(e.getCause().getMessage(), e.getCause());
		}
		return result == null ? "ok" : result.toString();
	}

	public ValidationErrors validate(Object component, LogicRequest context, ResourceBundle bundle)
			throws UnstableValidationException {
		ValidationErrors errors = new BasicValidationErrors();
		if (validateMethod != null) {
			try {
				validateMethod.invoke(component, errors);
			} catch (MethodInvocationException e) {
				LOG.error("Nasty validation method has thrown an exception!", e);
				throw new UnstableValidationException("Error during validation process", e);
			}
		}
		return errors;
	}

	public boolean shouldRedirect() {
		return this.shouldRedirect;
	}

	public JavaMethod getMetadata() {
		return method;
	}

	public List<MethodParameter> getParameters() {
		return parameters;
	}

}
