package org.vraptor.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicMethod;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.interceptor.ComponentLookupInterceptor;
import org.vraptor.interceptor.ExecuteLogicInterceptor;
import org.vraptor.interceptor.InjectionInterceptor;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.interceptor.InterceptorType;
import org.vraptor.interceptor.OutjectionInterceptor;
import org.vraptor.interceptor.RegisterAttributesInteceptor;
import org.vraptor.interceptor.SettingAndValidationInterceptor;
import org.vraptor.interceptor.ViewInterceptor;
import org.vraptor.url.SimpleLogicRequest;
import org.vraptor.view.ViewException;

/**
 * VRaptor's engine.
 * 
 * @author Guilherme Silveira
 * @author Paulo Silveira
 */
public class VRaptorExecution {

	private static final Logger LOG = Logger.getLogger(VRaptorExecution.class);

	private VRaptorController controller;

	private SimpleLogicRequest request;

	public VRaptorExecution(SimpleLogicRequest logicRequest, VRaptorController controller) {
		this.controller = controller;
		this.request = logicRequest;
	}

	public String execute() throws ComponentNotFoundException, LogicNotFoundException, ViewException,
			InterceptorInstantiationException, LogicException {
		if (LOG.isDebugEnabled()) {
			LOG.debug("Calling execute on " + request.toString());
		}

		ComponentType componentType = this.controller.getWebApplication().getComponentManager().getComponent(request);

		// gets the logic method
		LogicMethod method = componentType.getLogic(request.getLogicName());

		// TODO those interceptors should not use member variables to be faster:
		// cache them

		Interceptor register = new RegisterAttributesInteceptor(method.getMetadata());

		List<Interceptor> after = new ArrayList<Interceptor>();

		after.add(new ComponentLookupInterceptor(componentType));
		after
				.add(new InjectionInterceptor(this.request.getLogicContext(), controller.getIntrospector(),
						componentType));
		after.add(new SettingAndValidationInterceptor(componentType, controller.getIntrospector(), controller
				.getWebApplication().getConverterManager(), controller.getWebApplication().getViewManager(), method));
		after.add(new ExecuteLogicInterceptor(controller.getIntrospector(), componentType, method));
		after.add(new OutjectionInterceptor(controller.getIntrospector(), componentType));
		after.add(new ViewInterceptor(controller.getWebApplication().getViewManager(), method));

		List<Interceptor> interceptors = generateInterceptors(Collections.singletonList(register), componentType, after);

		InterceptorsLogicFlow logicFlow = new InterceptorsLogicFlow(request, controller.getIntrospector(), interceptors);

		logicFlow.execute();

		// returns its result, if any
		return logicFlow.getLogicRequest().getResult();
	}

	private static List<Interceptor> generateInterceptors(List<Interceptor> before, ComponentType componentType,
			List<Interceptor> after) throws InterceptorInstantiationException {
		List<Interceptor> interceptors = new ArrayList<Interceptor>();
		interceptors.addAll(before);
		for (InterceptorType type : componentType.getInterceptors()) {
			try {
				Interceptor interceptor = type.newInstance();
				if (LOG.isDebugEnabled()) {
					LOG.debug("Adding interceptor " + interceptor.getClass().getName());
				}
				interceptors.add(interceptor);
			} catch (ComponentInstantiationException e) {
				throw new InterceptorInstantiationException(e.getMessage(), e);
			}
		}
		interceptors.addAll(after);
		return interceptors;
	}

}
