package org.vraptor.interceptor;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.annotations.In;
import org.vraptor.annotations.InterceptedBy;
import org.vraptor.annotations.Out;
import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.component.FieldAnnotation;
import org.vraptor.reflection.ReflectionUtil;

/**
 * Interceptor class to deal with interceptors instances
 * 
 * @author Guilherme Silveira
 * @author Paulo Silveira
 */
public class InterceptorType {

	private static final Logger LOG = Logger.getLogger(InterceptorType.class);

	private static final ConcurrentMap<Class, InterceptorType> types = new ConcurrentHashMap<Class, InterceptorType>();

	private final Class<? extends Interceptor> clazz;

	/** Out annotations */
	private final List<FieldAnnotation<Out>> outAnnotations;

	/** In annotations */
	private final List<FieldAnnotation<In>> inAnnotations;

	/**
	 * New interceptor class
	 * 
	 * @param c
	 *            class
	 */
	private InterceptorType(Class<? extends Interceptor> c) {
		this.clazz = c;

		this.outAnnotations = ReflectionUtil.readAnnotations(this.clazz, Out.class);

		this.inAnnotations = ReflectionUtil.readAnnotations(this.clazz, In.class);
	}

	public Interceptor newInstance() throws ComponentInstantiationException {
		return ReflectionUtil.instantiate(this.clazz);
	}

	/**
	 * @return Returns the outAnnotations.
	 */
	public List<FieldAnnotation<Out>> getOutAnnotations() {
		return this.outAnnotations;
	}

	/**
	 * @return Returns the inAnnotations.
	 */
	public List<FieldAnnotation<In>> getInAnnotations() {
		return this.inAnnotations;
	}

	public Class getInterceptorClass() {
		return this.clazz;
	}

	/**
	 * Returns an InterceptorType which represents the given interceptor
	 * class.
	 * 
	 * @param c
	 * @return
	 */
	public static InterceptorType getType(Class<? extends Interceptor> c) {
		if (!types.containsKey(c)) {
			LOG.debug("creating interceptor type for " + c);
			types.put(c, new InterceptorType(c));
		}
		return types.get(c);
	}
	
	/**
	 * Returns an InterceptorType which represents the given interceptor
	 * 
	 * @param interceptor
	 * @return
	 */
	public static InterceptorType getType(Interceptor interceptor) {
		return getType(interceptor.getClass());
	}

	

	/**
	 * Returns a list of interceptors for the specified component class,
	 * parsing its InterceptedBy annotations
	 * 
	 * @param clazz
	 *            class
	 * @return interceptor type list
	 */
	public static List<InterceptorType> getInterceptors(Class<?> clazz) {
		List<InterceptorType> types = new ArrayList<InterceptorType>();
		if (clazz.isAnnotationPresent(InterceptedBy.class)) {
			@SuppressWarnings("unchecked")
			InterceptedBy annotation = (InterceptedBy) clazz.getAnnotation(InterceptedBy.class);
			for (Class<? extends Interceptor> c : annotation.value()) {
				InterceptorType type = InterceptorType.getType(c);
				types.add(type);
			}
		}
		return types;
	}

}
