package org.vraptor.introspector;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.log4j.Logger;
import org.vraptor.LogicRequest;
import org.vraptor.annotations.In;
import org.vraptor.annotations.Out;
import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.component.FieldAnnotation;
import org.vraptor.converter.ConversionException;
import org.vraptor.converter.ConverterManager;
import org.vraptor.i18n.ValidationMessage;
import org.vraptor.reflection.GettingException;
import org.vraptor.reflection.JPathExecutor;
import org.vraptor.reflection.MethodInvocationException;
import org.vraptor.reflection.ReflectionUtil;
import org.vraptor.reflection.SettingException;
import org.vraptor.scope.RequestContext;
import org.vraptor.scope.ScopeType;

/**
 * The default introspector implementation.
 * 
 * @author Guilherme Silveira
 */
public class BasicIntrospector implements Introspector {

	private static final Logger LOG = Logger.getLogger(BasicIntrospector.class);

	private static final Object[] ZERO_LENGTH_ARRAY = null;

	private final KeyExtractor keyExtractor = new KeyExtractor();

	private Set<Parameter> addAll(Set<String> keys) {
		Set<Parameter> ts = new TreeSet<Parameter>();
		for (String key : keys) {
			ts.add(new Parameter(key));
		}
		return ts;
	}

	@SuppressWarnings("unchecked")
	public List<ValidationMessage> readParameters(List<ReadParameter> parametersToRead, Object component,
			LogicRequest logicRequest, ConverterManager converterManager, Object[] methodParamObjects)
			throws SettingException {

		JPathExecutor executor = new JPathExecutor(converterManager, logicRequest, methodParamObjects, component);
		RequestContext request = logicRequest.getRequestContext();

		Map<String, Object> parameters = request.getParameterMap();
		List<ValidationMessage> problems = new ArrayList<ValidationMessage>();

		out: for (Parameter p : addAll(parameters.keySet())) {
			String parameter = p.getKey();
			// String path[] = parameter.split("[\\.\\[\\]]+");
			for (ReadParameter read : parametersToRead) {
				// tries to fill
				if (!p.matches(read.getKey())) {
					continue;
				}
				if (LOG.isDebugEnabled()) {
					LOG.debug("Parameter " + parameter + " will be used on field " + read.getKey());
				}
				try {
					// TODO: dont do this, cache it in request instead (or
					// use the
					// internal request itself)
					executor.set(p.getPath(), singleValue(parameters.get(parameter)), arrayValue(parameters
							.get(parameter)), read);
				} catch (ConversionException e) {
					// validation problem...
					if (LOG.isDebugEnabled()) {
						LOG.debug(e.getMessage(), e);
					}
					problems.add(e.getI18NMessage());
				}
				continue out;
			}
			if (LOG.isDebugEnabled()) {
				LOG.debug("Parameter not used: " + parameter);
			}
		}

		return problems;

	}

	private String[] arrayValue(Object val) {
		if (val.getClass().isArray()) {
			return (String[]) val;
		}
		return new String[] { (String) val };
	}

	private String singleValue(Object val) {
		if (val.getClass().isArray()) {
			return (String) Array.get(val, 0);
		}
		return (String) val;
	}

	public void inject(List<FieldAnnotation<In>> inAnnotations, Object component, LogicRequest request)
			throws ComponentInstantiationException, SettingException {

		for (FieldAnnotation<In> info : inAnnotations) {
			// tries to fill
			In in = info.getAnnotation();
			String key = keyExtractor.extractInKey(info);
			Object value = in.scope().getContext(request).getAttribute(key);

			setFieldForInjection(component, info, in, key, value);
		}

	}

	private void setFieldForInjection(Object component, FieldAnnotation<In> info, In in, String key, Object value)
			throws ComponentInstantiationException, SettingException {

		if (LOG.isDebugEnabled()) {
			LOG.debug("Injecting with key " + key + " from " + in.scope());
		}

		if (value == null) {
			if (in.create()) {
				if (LOG.isDebugEnabled()) {
					LOG.debug("Trying to create " + info.getField().getType().getName());
				}
				value = ReflectionUtil.instantiate(info.getField().getType());
			} else if (in.required()) {
				throw new SettingException("Unable to fill inject value for field " + info.getField().getName());
			} else {
				return;
			}
		}

		ReflectionUtil.setField(component, info.getField(), value);
	}

	/**
	 * @see org.vraptor.introspector.Introspector#outject(java.util.List,
	 *      java.lang.Object, org.vraptor.scope.LogicRequest)
	 */
	public void outject(List<FieldAnnotation<Out>> outAnnotations, Object component, LogicRequest request)
			throws GettingException {
		for (FieldAnnotation<Out> info : outAnnotations) {
			// tries to fill
			Out out = info.getAnnotation();
			String key = keyExtractor.extractOutKey(info);
			Object value = ReflectionUtil.get(component, info.getField());
			if (LOG.isDebugEnabled()) {
				LOG.debug("Outjecting " + info.getField().getName() + " with key " + key + " at " + out.scope());
			}
			out.scope().getContext(request).setAttribute(key, value);
		}

		Map<String, Method> getters = ReflectionUtil.getGetters(component.getClass());
		// TODO: cache the getters for each ComponentType
		for (String key : getters.keySet()) {
			Object value;
			try {
				value = ReflectionUtil.invoke(component, getters.get(key), ZERO_LENGTH_ARRAY);
			} catch (MethodInvocationException e) {
				throw new GettingException(e.getMessage(), e);
			}
			if (LOG.isDebugEnabled()) {
				LOG.debug(String.format("Outjecting %s via getter %s, value %s", key, getters.get(key), value));
			}
			// TODO: make possible to associate a getter to outject to a
			// different scope, instead of hardcoding ScopeType.REQUEST here

			ScopeType.REQUEST.getContext(request).setAttribute(key, value);
		}
	}

}
